@@ -16,6 +16,7 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
 #include <errno.h>
 #include <string.h>
 #include "kernel-lib/bitops.h"
@@ -79,7 +80,8 @@ static unsigned int leaf_data_end(const struct extent_buffer *leaf)
  * have to adjust any offsets to account for the header in the leaf.  This
  * handles that math to simplify the callers.
  */
-static inline void memmove_leaf_data(const struct extent_buffer *leaf,
+__maybe_unused
+static inline void memmove_leaf_data(struct extent_buffer *leaf,
 				     unsigned long dst_offset,
 				     unsigned long src_offset,
 				     unsigned long len)
@@ -102,7 +104,8 @@ static inline void memmove_leaf_data(const struct extent_buffer *leaf,
  * have to adjust any offsets to account for the header in the leaf.  This
  * handles that math to simplify the callers.
  */
-static inline void copy_leaf_data(const struct extent_buffer *dst,
+__maybe_unused
+static inline void copy_leaf_data(struct extent_buffer *dst,
 				  const struct extent_buffer *src,
 				  unsigned long dst_offset,
 				  unsigned long src_offset, unsigned long len)
@@ -122,7 +125,8 @@ static inline void copy_leaf_data(const struct extent_buffer *dst,
  * Wrapper around memmove_extent_buffer() that does the math to get the
  * appropriate offsets into the leaf from the item numbers.
  */
-static inline void memmove_leaf_items(const struct extent_buffer *leaf,
+__maybe_unused
+static inline void memmove_leaf_items(struct extent_buffer *leaf,
 				      int dst_item, int src_item, int nr_items)
 {
 	memmove_extent_buffer(leaf, btrfs_item_nr_offset(leaf, dst_item),
@@ -142,7 +146,8 @@ static inline void memmove_leaf_items(const struct extent_buffer *leaf,
  * Wrapper around copy_extent_buffer() that does the math to get the
  * appropriate offsets into the leaf from the item numbers.
  */
-static inline void copy_leaf_items(const struct extent_buffer *dst,
+__maybe_unused
+static inline void copy_leaf_items(struct extent_buffer *dst,
 				   const struct extent_buffer *src,
 				   int dst_item, int src_item, int nr_items)
 {
@@ -1241,6 +1246,17 @@ static void reada_for_search(struct btrfs_fs_info *fs_info,
 	}
 }
 
+/*
+ * Find the first key in @fs_root that matches all the following conditions:
+ *
+ * - key.obojectid == @iobjectid
+ * - key.type == @key_type
+ * - key.offset >= ioff
+ *
+ * Return 0 if such key can be found, and @found_key is updated.
+ * Return >0 if no such key can be found.
+ * Return <0 for critical errors.
+ */
 int btrfs_find_item(struct btrfs_root *fs_root, struct btrfs_path *found_path,
 		u64 iobjectid, u64 ioff, u8 key_type,
 		struct btrfs_key *found_key)
@@ -1274,11 +1290,10 @@ int btrfs_find_item(struct btrfs_root *fs_root, struct btrfs_path *found_path,
 	}
 
 	btrfs_item_key_to_cpu(eb, found_key, path->slots[0]);
-	if (found_key->type != key.type ||
-			found_key->objectid != key.objectid) {
+	if (found_key->type != key.type || found_key->objectid != key.objectid)
 		ret = 1;
-		goto out;
-	}
+	else
+		ret = 0;
 
 out:
 	if (path != found_path)
@@ -1872,9 +1887,7 @@ int btrfs_leaf_free_space(const struct extent_buffer *leaf)
 	u32 leaf_data_size;
 	int ret;
 
-	BUG_ON(!leaf->fs_info);
-	BUG_ON(leaf->fs_info->nodesize != leaf->len);
-	leaf_data_size = BTRFS_LEAF_DATA_SIZE(leaf->fs_info);
+	leaf_data_size = __BTRFS_LEAF_DATA_SIZE(leaf->len);
 	ret = leaf_data_size - leaf_space_used(leaf, 0 ,nritems);
 	if (ret < 0) {
 		printk("leaf free space ret %d, leaf data size %u, used %d nritems %d\n",
@@ -2450,7 +2463,7 @@ int btrfs_split_item(struct btrfs_trans_handle *trans,
 	u32 nritems;
 	u32 orig_offset;
 	struct btrfs_disk_key disk_key;
-	char *buf;
+	char *buf = NULL;
 
 	leaf = path->nodes[0];
 	btrfs_item_key_to_cpu(leaf, &orig_key, path->slots[0]);
@@ -2469,11 +2482,13 @@ int btrfs_split_item(struct btrfs_trans_handle *trans,
 	/* if our item isn't there or got smaller, return now */
 	if (ret != 0 || item_size != btrfs_item_size(path->nodes[0],
 							path->slots[0])) {
-		return -EAGAIN;
+		ret = -EAGAIN;
+		goto error;
 	}
 
 	ret = split_leaf(trans, root, &orig_key, path, 0, 0);
-	BUG_ON(ret);
+	if (ret < 0)
+		goto error;
 
 	BUG_ON(btrfs_leaf_free_space(leaf) < sizeof(struct btrfs_item));
 	leaf = path->nodes[0];
@@ -2484,7 +2499,10 @@ split:
 
 
 	buf = kmalloc(item_size, GFP_NOFS);
-	BUG_ON(!buf);
+	if (!buf) {
+		ret = -ENOMEM;
+		goto error;
+	}
 	read_extent_buffer(leaf, buf, btrfs_item_ptr_offset(leaf,
 			    path->slots[0]), item_size);
 	slot = path->slots[0] + 1;
@@ -2530,6 +2548,10 @@ split:
 	}
 	kfree(buf);
 	return ret;
+error:
+	kfree(buf);
+	btrfs_release_path(path);
+	return ret;
 }
 
 void btrfs_truncate_item(struct btrfs_path *path, u32 new_size, int from_end)
