@@ -16,22 +16,25 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
 #include <stdbool.h>
-#include "kerncompat.h"
-#include "kernel-shared/extent_io.h"
+#include <string.h>
+#include <errno.h>
 #include "kernel-lib/list.h"
 #include "kernel-lib/raid56.h"
 #include "kernel-lib/bitmap.h"
+#include "kernel-shared/accessors.h"
+#include "kernel-shared/extent-io-tree.h"
+#include "kernel-shared/extent_io.h"
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/volumes.h"
 #include "kernel-shared/disk-io.h"
 #include "kernel-shared/messages.h"
+#include "kernel-shared/uapi/btrfs.h"
+#include "kernel-shared/uapi/btrfs_tree.h"
+#include "common/messages.h"
 #include "common/utils.h"
 #include "common/device-utils.h"
 #include "common/internal.h"
@@ -172,7 +175,7 @@ static void free_extent_buffer_final(struct extent_buffer *eb)
 		BUG_ON(eb->fs_info->cache_size < eb->len);
 		eb->fs_info->cache_size -= eb->len;
 	}
-	free(eb);
+	kfree(eb);
 }
 
 static void free_extent_buffer_internal(struct extent_buffer *eb, bool free_now)
@@ -278,7 +281,7 @@ struct extent_buffer *alloc_extent_buffer(struct btrfs_fs_info *fs_info,
 			return NULL;
 		ret = insert_cache_extent(&fs_info->extent_cache, &eb->cache_node);
 		if (ret) {
-			free(eb);
+			kfree(eb);
 			return NULL;
 		}
 		list_add_tail(&eb->lru, &fs_info->lru);
@@ -336,13 +339,12 @@ static int read_raid56(struct btrfs_fs_info *fs_info, void *buf, u64 logical,
 	ASSERT(len <= BTRFS_STRIPE_LEN);
 
 	pointers = calloc(num_stripes, sizeof(void *));
-	if (!pointers) {
-		ret = -ENOMEM;
-		goto out;
-	}
+	if (!pointers)
+		return -ENOMEM;
+
 	/* Allocate memory for the full stripe */
 	for (i = 0; i < num_stripes; i++) {
-		pointers[i] = malloc(BTRFS_STRIPE_LEN);
+		pointers[i] = kmalloc(BTRFS_STRIPE_LEN, GFP_KERNEL);
 		if (!pointers[i]) {
 			ret = -ENOMEM;
 			goto out;
@@ -411,10 +413,10 @@ static int read_raid56(struct btrfs_fs_info *fs_info, void *buf, u64 logical,
 			BTRFS_STRIPE_LEN, len);
 	ret = 0;
 out:
-	free(failed_stripe_bitmap);
+	kfree(failed_stripe_bitmap);
 	for (i = 0; i < num_stripes; i++)
-		free(pointers[i]);
-	free(pointers);
+		kfree(pointers[i]);
+	kfree(pointers);
 	return ret;
 }
 
@@ -439,12 +441,12 @@ int read_data_from_disk(struct btrfs_fs_info *info, void *buf, u64 logical,
 	if (mirror > 1 && multi->type & BTRFS_BLOCK_GROUP_RAID56_MASK) {
 		ret = read_raid56(info, buf, logical, read_len, mirror, multi,
 				  raid_map);
-		free(multi);
-		free(raid_map);
+		kfree(multi);
+		kfree(raid_map);
 		*len = read_len;
 		return ret;
 	}
-	free(raid_map);
+	kfree(raid_map);
 	device = multi->stripes[0].dev;
 
 	if (device->fd <= 0) {
@@ -509,7 +511,7 @@ int write_data_to_disk(struct btrfs_fs_info *info, const void *buf, u64 offset,
 			this_len = min(this_len, bytes_left);
 			this_len = min(this_len, (u64)info->nodesize);
 
-			eb = malloc(sizeof(struct extent_buffer) + this_len);
+			eb = kmalloc(sizeof(struct extent_buffer) + this_len, GFP_KERNEL);
 			if (!eb) {
 				error_msg(ERROR_MSG_MEMORY, "extent buffer");
 				ret = -ENOMEM;
@@ -525,7 +527,7 @@ int write_data_to_disk(struct btrfs_fs_info *info, const void *buf, u64 offset,
 						       stripe_len, raid_map);
 			BUG_ON(ret < 0);
 
-			free(eb);
+			kfree(eb);
 			kfree(raid_map);
 			raid_map = NULL;
 		} else while (dev_nr < multi->num_stripes) {
@@ -585,7 +587,8 @@ int set_extent_buffer_dirty(struct extent_buffer *eb)
 	return 0;
 }
 
-int btrfs_clear_buffer_dirty(struct extent_buffer *eb)
+int btrfs_clear_buffer_dirty(struct btrfs_trans_handle *trans,
+			     struct extent_buffer *eb)
 {
 	struct extent_io_tree *tree = &eb->fs_info->dirty_buffers;
 	if (eb->flags & EXTENT_BUFFER_DIRTY) {
@@ -609,30 +612,30 @@ void read_extent_buffer(const struct extent_buffer *eb, void *dst,
 	memcpy(dst, eb->data + start, len);
 }
 
-void write_extent_buffer_fsid(const struct extent_buffer *eb, const void *src)
+void write_extent_buffer_fsid(struct extent_buffer *eb, const void *src)
 {
 	write_extent_buffer(eb, src, btrfs_header_fsid(), BTRFS_FSID_SIZE);
 }
 
-void write_extent_buffer_chunk_tree_uuid(const struct extent_buffer *eb,
+void write_extent_buffer_chunk_tree_uuid(struct extent_buffer *eb,
 		const void *src)
 {
 	write_extent_buffer(eb, src, btrfs_header_chunk_tree_uuid(eb), BTRFS_FSID_SIZE);
 }
 
-void write_extent_buffer(const struct extent_buffer *eb, const void *src,
+void write_extent_buffer(struct extent_buffer *eb, const void *src,
 			 unsigned long start, unsigned long len)
 {
 	memcpy((void *)eb->data + start, src, len);
 }
 
-void copy_extent_buffer_full(const struct extent_buffer *dst,
+void copy_extent_buffer_full(struct extent_buffer *dst,
 			     const struct extent_buffer *src)
 {
 	copy_extent_buffer(dst, src, 0, 0, src->len);
 }
 
-void copy_extent_buffer(const struct extent_buffer *dst,
+void copy_extent_buffer(struct extent_buffer *dst,
 			const struct extent_buffer *src,
 			unsigned long dst_offset, unsigned long src_offset,
 			unsigned long len)
@@ -640,13 +643,19 @@ void copy_extent_buffer(const struct extent_buffer *dst,
 	memcpy((void *)dst->data + dst_offset, src->data + src_offset, len);
 }
 
-void memmove_extent_buffer(const struct extent_buffer *dst, unsigned long dst_offset,
+void memcpy_extent_buffer(struct extent_buffer *dst, unsigned long dst_offset,
+			  unsigned long src_offset, unsigned long len)
+{
+	memcpy((void *)dst->data + dst_offset, dst->data + src_offset, len);
+}
+
+void memmove_extent_buffer(struct extent_buffer *dst, unsigned long dst_offset,
 			   unsigned long src_offset, unsigned long len)
 {
 	memmove((void *)dst->data + dst_offset, dst->data + src_offset, len);
 }
 
-void memset_extent_buffer(const struct extent_buffer *eb, char c,
+void memset_extent_buffer(struct extent_buffer *eb, char c,
 			  unsigned long start, unsigned long len)
 {
 	memset((void *)eb->data + start, c, len);
