@@ -7,7 +7,9 @@ typedef enum
 {
 	CSM_NONE  = 0x00,
 	CSM_CRC32 = 0x20,
+#ifdef HAVE_XXHASH
 	CSM_XXH64 = 0x30,
+#endif
 	CSM_CRC32_0001 = 0xFF, // use crc32_0001() and watch for x64 bug
 } checksum_mode_enum;
 
